//
//  MailDeliveryConsole.m
//  MailDelivery
//
//  Created by Dante Palacios on 5/10/10.
//  Copyright 2010 Dante Palacios. All rights reserved.
//

#import <AppKit/NSWindow.h>
#import <AppKit/NSTextView.h>

#import "MailDeliveryConsole.h"

#import "Macros.h"
#import "SynthesizeSingleton.h"


@implementation MailDeliveryConsole


SYNTHESIZE_SINGLETON_FOR_CLASS(MailDeliveryConsole)

- (id)init
{
	self = [super init];
	if (self != nil)
	{
		// a Webview or a NSTextview do their part very well, but we must try to play with their rules....
		[self performSelectorOnMainThread:@selector(load) withObject:nil waitUntilDone:YES];
	}
	return self;
}

- (void)dealloc 
{
	
	[super dealloc];
}

- (void)load 
{
	if (!window && [NSBundle loadNibNamed:NSStringFromClass([self class]) owner:self])
	{
		if (![window frameAutosaveName])
		{
			//[window center];
			[window setFrameAutosaveName:NSStringFromClass([self class])];
		}
		
		window.delegate = self;
		
		[window setContentBorderThickness:24.0 forEdge:NSMinYEdge];
		
		[logView setFont:[NSFont userFixedPitchFontOfSize:11.0f]];
		[logView setString:@""];
	}
}

- (IBAction)clear:(id)sender;
{
	[self performSelectorOnMainThread:@selector(_clear) withObject:nil waitUntilDone:YES];
}

- (void)open;
{
	if (self.isLoggingEnabled)
	{
		[window makeKeyAndOrderFront:self];
	}
}

- (void)close; {
	[window orderOut:self];
}

- (void)log:(NSString *)string;
{
	if (self.isLoggingEnabled && string.length)
	{
		[self performSelectorOnMainThread:@selector(_log:) withObject:string waitUntilDone:YES];
	}
}

#pragma mark private methods

- (void)_clear
{
	[logView setString:@""];
}

- (void)_log:(NSString *)string;
{
	NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
	[dateFormatter setFormatterBehavior:NSDateFormatterBehavior10_4];
	[dateFormatter setDateStyle:NSDateFormatterShortStyle];
	[dateFormatter setTimeStyle:NSDateFormatterMediumStyle];
	
	NSString *dateString = [dateFormatter stringFromDate:[NSDate date]];
	//NSMutableString *currentText = [NSMutableString string];
	//[currentText appendString:[[logView textStorage] string]];
	//[currentText appendFormat:@"%@ %@\r\n", dateString, string];
	//[logView setString:currentText];
	
    [logView replaceCharactersInRange:NSMakeRange([[logView textStorage] length], 0) withString:[NSString stringWithFormat:@"%@ %@\r\n", dateString, string]];
	
	[dateFormatter release];
}

#pragma mark getters & setters

- (NSString *)windowTitle
{
    return [window title];
}

- (void)setWindowTitle:(NSString *)value
{
    [window setTitle:value];
}

- (BOOL)isLoggingEnabled
{
    return [[NSUserDefaults standardUserDefaults] boolForKey:@"IsLoggingEnabled"];
}

- (void)setLoggingEnabled:(BOOL)value
{
    [[NSUserDefaults standardUserDefaults] setBool:value forKey:@"IsLoggingEnabled"];
}

@end
